% Wireless Communication & Signal Processing Group
% www.wcsp.eng.usf.edu
% Update 1: Alphan Sahin, Ertugrul Guvenkaya, 03/16/13

% This code requires Matlab Instrument Control Toolbox and
% WaveformDownloadAssistant function package which is available in Agilent
% website

%% Presetup
close all
clear all
clc
disp(['Welcome to WCSP Receiver...'])
disp([''])
disp([''])

addpath(genpath('tools'))
%% Script Parameters
global parametersReceiver;                        	% All receiver parameters.
defaults;

% Figure parameters
mainFigure = 1;
figure(mainFigure)
handle1 = subplot(241);
handle2 = subplot(242);
handle3 = subplot(243);
handle4 = subplot(245);
handle5 = subplot(246);
handle6 = subplot(247);
handle7 = subplot(244);
handle8 = subplot(248);

indexStartMaxMinInterval = 300; % frame will be away from the first sample and last sample with this range.

%% Initialize Receiver
receiverObject = receiverInitialization;

while(1)
    reTryFlag = 1;
    minimumLimit = indexStartMaxMinInterval;
    maximumLimit = (floor(parametersReceiver.measureTime/(1/parametersReceiver.sample_rate))-(length(parametersReceiver.header))*parametersReceiver.oversampling_ratio)-indexStartMaxMinInterval;
    while (reTryFlag) % this is because coarse synch is ok for fine synch in the following steps.
        disp(['Status: Capturing signal...'])
        
        %% Capture signal
        [receivedSignal, time] = receiveData(receiverObject);

        disp(['Status: Time and frequency domain observation...'])
        figure(mainFigure), subplot(handle1)
        plot(time, real(receivedSignal))
        grid on
        title('Received Signal (Real)')
        xlabel('Time (s)')
        ylabel('Magnitude (V)')
        xlim([0 time(end)])
        
        figure(mainFigure), subplot(handle2)
        plot(20*log10(abs(fftshift(fft(receivedSignal)/sqrt(length(receivedSignal)))))+10) % dBm, check basic mode guide. Don't forget to take the 50 ohms into account
        grid on
        ylim([-130, 0])
        title('Spectrum')
        xlabel('Bin')
        ylabel('Magnitude(dBm)')
        drawnow % refresh the figures
        
        %% Filter the other signals beside the base band signal
        % If you signal power is weak, other signal might dominate yur
        % signal. In this part, these components (outside of the desired band)
        % are suppressed via an ideal low pass filter.
        receivedSignal_filtered = initialFiltering(receivedSignal);
        
        figure(mainFigure), subplot(handle2)
        hold on
        plot(20*log10(abs(fftshift(fft(receivedSignal_filtered)/sqrt(length(receivedSignal)))))+10,'-g')
        hold off
        drawnow % refresh the figures   
        grid on


        %% Match Filtering 
        % Note that RRC filter is the filter that generates a Nyquist
        % filter. We need Nyquist filter to avoid ISI. Also, it provides
        % match filtering so that SNR is maximized.
        receivedSignal_match = matchFiltering(receivedSignal_filtered);

        %% Coarse Synchronization - Correlator
        % Estimationg the beginning of the symbol
        [correlationOutput indexStart] = coarseSynchronization(receivedSignal_match);
        figure(mainFigure), subplot(handle4)
        plot(time(1:end-((2^parametersReceiver.frame_seq_no-1) * parametersReceiver.oversampling_ratio)),abs(correlationOutput))
        title('Correlator Output')
        xlabel('Time(s)')
        ylabel('Correlation')
        grid on
        xlim([0 time(end)])
    
        if indexStart >  minimumLimit && indexStart < maximumLimit
            reTryFlag = 0; % ready for fine synchronization
        end
    end
    

    figure(mainFigure), subplot(handle1)
    hold on
    plot(time(indexStart:indexStart+parametersReceiver.msequenceLengthSample), real(receivedSignal(indexStart:indexStart+parametersReceiver.msequenceLengthSample)), '-r')
    plot(time(indexStart+parametersReceiver.msequenceLengthSample:indexStart+2*parametersReceiver.msequenceLengthSample), real(receivedSignal(indexStart+parametersReceiver.msequenceLengthSample:indexStart+2*parametersReceiver.msequenceLengthSample)), '-g')
    hold off    
    % Get useful part of the signal for faster emulation
    receivedSignal_usefulpart = receivedSignal_match(indexStart-indexStartMaxMinInterval+1:indexStart+parametersReceiver.headerLengthSample+indexStartMaxMinInterval-1);
    indexStart = indexStartMaxMinInterval+1;
    
    %% Frequency Offset Correction
    [receivedSignal_offcor] = frequencyOffsetCompensation_v2(receivedSignal_usefulpart, indexStart);
    %% Phase Offset Correction
    [receivedSignal_offcor] = phaseOffsetCompensation(receivedSignal_offcor, indexStart);

    eyediagram(real(receivedSignal_offcor(indexStart:indexStart+length(parametersReceiver.msequence)*parametersReceiver.oversampling_ratio)),2*parametersReceiver.oversampling_ratio, 1,0,'-r',mainFigure,handle3, [-max(max(real(receivedSignal_offcor)),max(imag(receivedSignal_offcor))) max(max(real(receivedSignal_offcor)),max(imag(receivedSignal_offcor)))] );
    title('Eyediagram of Real Part')
    xlabel('Samples')
    ylabel('Magnitude')
    grid on
    eyediagram(imag(receivedSignal_offcor(indexStart:indexStart+length(parametersReceiver.msequence)*parametersReceiver.oversampling_ratio)),2*parametersReceiver.oversampling_ratio, 1,0,'-r',mainFigure,handle6, [-max(max(real(receivedSignal_offcor)),max(imag(receivedSignal_offcor))) max(max(real(receivedSignal_offcor)),max(imag(receivedSignal_offcor)))]);
    title('Eyediagram of Imag Part')
    xlabel('Samples')
    ylabel('Magnitude')
    grid on
    figure(1), subplot(handle5)
    plot(receivedSignal_offcor(indexStart:indexStart+length(parametersReceiver.msequence)*parametersReceiver.oversampling_ratio), '.')
    title('Constellation after CFO Compensation')
    xlabel('Real')
    ylabel('Imag')
    
    ylim([-max(abs(receivedSignal_offcor)), max(abs(receivedSignal_offcor))])
    xlim([-max(abs(receivedSignal_offcor)), max(abs(receivedSignal_offcor))])
    grid on
    
    drawnow % refresh the figures

    %% Channel Estimation       
    possibleNumberOfTaps = [30];
    scanRangeLimitForFineSynch = 30; % +,-

    [bestChannel, ~] = channelEstimation_Student(receivedSignal_offcor, indexStart, possibleNumberOfTaps, scanRangeLimitForFineSynch);
    figure(mainFigure), subplot(handle7)
    plot(10*log10(abs(bestChannel).^2/sum(abs(bestChannel).^2)), '-o')
    grid on;
    title(['CIR - Your Correlator'])
    ylabel('Power');
    xlabel('Taps')
    ylim([-80,0])    

    [bestChannel, ~,k] = channelEstimation_SlidingCorrelator(receivedSignal_offcor, indexStart, possibleNumberOfTaps, scanRangeLimitForFineSynch);
    
    figure(mainFigure), subplot(handle5)
    hold on
    receivedSignal_downsampled=downsample(receivedSignal_offcor(indexStart-k : indexStart-k+(length(parametersReceiver.header))*parametersReceiver.oversampling_ratio-1),parametersReceiver.oversampling_ratio).';
    plot(receivedSignal_downsampled, '.r')
    hold off

    figure(mainFigure), subplot(handle8)
    plot(10*log10(abs(bestChannel).^2/sum(abs(bestChannel).^2)), '-o')
    grid on;
    ylabel('Power');
    xlabel('Taps')
    title(['CIR - Sliding Correlator'])
    ylim([-80,0])   

    drawnow % refresh the figures
    disp(['Status: Completed...'])
    period = toc; 
    disp(['Matlab processing rate is ' num2str(length(receivedSignal)/period)])
    disp(['----------------------------------------------------'])
end